/****************************************************************************
** Form interface generated from reading ui file './runoptionswidgetbase.ui'
**
** Created: Fri Jul 29 04:10:22 2005
**      by: The User Interface Compiler ($Id: qt/main.cpp   3.3.4   edited Nov 24 2003 $)
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef RUN_OPTIONS_WIDGET_H
#define RUN_OPTIONS_WIDGET_H

#include <qvariant.h>
#include <qwidget.h>
#include <ksqueezedtextlabel.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QButtonGroup;
class QRadioButton;
class KSqueezedTextLabel;
class QLineEdit;
class QPushButton;
class QGroupBox;
class QLabel;
class QCheckBox;

class RunOptionsWidgetBase : public QWidget
{
    Q_OBJECT

public:
    RunOptionsWidgetBase( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~RunOptionsWidgetBase();

    QButtonGroup* directoryButtonGroup;
    QRadioButton* executableDirectory_radio;
    QRadioButton* buildDirectory_radio;
    KSqueezedTextLabel* buildDirectory_label;
    QRadioButton* customDirectory_radio;
    QLineEdit* customRunDirectory_edit;
    QPushButton* browseCustomButton;
    QGroupBox* programGroupBox;
    QLabel* mainprogram_label;
    QLabel* mainProgram_relativeness_label;
    QLineEdit* mainprogram_edit;
    QPushButton* browseMainButton;
    QLabel* progargs_label;
    QLineEdit* progargs_edit;
    QGroupBox* env_var_group;
    QCheckBox* startinterminal_box;
    QCheckBox* autocompile_box;

public slots:
    virtual void directoryRadioChanged();
    virtual void browseCustomDirectory();
    virtual void browseMainProgram();

protected:
    QVBoxLayout* run_options_widgetLayout;
    QGridLayout* directoryButtonGroupLayout;
    QHBoxLayout* layout2;
    QGridLayout* programGroupBoxLayout;
    QHBoxLayout* layout1;

protected slots:
    virtual void languageChange();

};

#endif // RUN_OPTIONS_WIDGET_H
