/****************************************************************************
** Form interface generated from reading ui file './settingswidget.ui'
**
** Created: Fri Jul 29 04:19:13 2005
**      by: The User Interface Compiler ($Id: qt/main.cpp   3.3.4   edited Nov 24 2003 $)
**
** WARNING! All changes made in this file will be lost!
****************************************************************************/

#ifndef SETTINGS_WIDGET_H
#define SETTINGS_WIDGET_H

#include <qvariant.h>
#include <qwidget.h>
#include <kfontdialog.h>
#include <qfont.h>
#include <qpushbutton.h>

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QCheckBox;
class QLabel;
class KURLRequester;
class QGroupBox;
class QPushButton;
class QButtonGroup;
class QRadioButton;
class QLineEdit;

class SettingsWidget : public QWidget
{
    Q_OBJECT

public:
    SettingsWidget( QWidget* parent = 0, const char* name = 0, WFlags fl = 0 );
    ~SettingsWidget();

    QCheckBox* lastProjectCheckbox;
    QLabel* textLabel1;
    KURLRequester* projects_url;
    QGroupBox* GroupBox1;
    QLabel* buttonLabel;
    QPushButton* changeMessageFontButton;
    QCheckBox* lineWrappingCheckBox;
    QCheckBox* dirNavigMsgCheckBox;
    QButtonGroup* compilerOutputButtonGroup;
    QRadioButton* veryShortCompilerOutputRadioButton;
    QRadioButton* shortCompilerOutputRadioButton;
    QRadioButton* fullCompilerOutputRadioButton;
    QGroupBox* GroupBox2;
    QLabel* buttonLabel_2;
    QPushButton* changeApplicationFontButton;
    QGroupBox* groupBox3;
    QCheckBox* embedDesignerCheckBox;
    QButtonGroup* buttonGroup3;
    QRadioButton* useOtherTerminal;
    QLineEdit* terminalEdit;
    QRadioButton* useKDETerminal;

public slots:
    virtual void changeMessageFontSlot();
    virtual void changeApplicationFontSlot();
    virtual QFont applicationFont();
    virtual QFont messageFont();
    virtual void setMessageFont( const QFont & fnt );
    virtual void setApplicationFont( const QFont & fnt );

protected:
    QFont _messageFont;
    QFont _applicationFont;

    QVBoxLayout* settings_widgetLayout;
    QHBoxLayout* GroupBox1Layout;
    QVBoxLayout* layout6;
    QHBoxLayout* Layout9;
    QSpacerItem* Spacer2;
    QVBoxLayout* compilerOutputButtonGroupLayout;
    QVBoxLayout* GroupBox2Layout;
    QHBoxLayout* Layout10;
    QSpacerItem* Spacer3;
    QGridLayout* groupBox3Layout;
    QVBoxLayout* buttonGroup3Layout;
    QGridLayout* layout7;

protected slots:
    virtual void languageChange();

};

#endif // SETTINGS_WIDGET_H
