<!DOCTYPE kpartgui>
<kpartplugin name="fullscreen" library="libfullscreenplugin" version="3">
<MenuBar>
  <Menu name="view">
    <Text>&amp;View</Text>
    <Action name="fullscreen"/>
 </Menu>
</MenuBar>
<ToolBar name="mainToolBar">
  <Action name="fullscreen" group="zoom_operations"/>
</ToolBar>
</kpartplugin>
