<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="KDevClassView" version="8">
<MenuBar>
  <Menu name="project" >
      <Action name="inheritance_dia" group="project_classes" />
  </Menu>
</MenuBar>

<ToolBar name="browserToolBar">
  <text>Browser Toolbar</text>
  <Separator/>
  <Action name="functionsnav_combo" />
</ToolBar>
</kpartgui>

