<!DOCTYPE kpartgui>
<kpartplugin name="KDevAStyle" library="libkdevastyle" version="2">
<MenuBar>
 <Menu name="edit">
  <Action name="edit_astyle" group="edit_astyle"/>
 </Menu>
</MenuBar>
</kpartplugin>
