<!DOCTYPE kpartgui>
<kpartplugin name="kdevprojectmanager" library="libkdevprojectmanager" version="2">
<MenuBar>
 <Menu name="build"><Text>&amp;Build</Text>
  <Action name="project_buildall"/>
  <Action name="project_build"/>
  <Separator />
  <Action name="project_refresh"/>
 </Menu>
</MenuBar>
</kpartplugin>
