<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="qeditor_part" version="13">
<MenuBar noMerge="1">

  <Menu name="file" noMerge="1"><text>&amp;File</text>
    <Action name="file_save" group="save_merge" />
    <Action name="file_save_as" group="save_merge" />
    <Action name="file_reload" group="revert_merge"/>
  </Menu>

  <Menu name="edit" noMerge="1"><text>&amp;Edit</text>
    <Action name="edit_undo" group="edit_undo_merge" />
    <Action name="edit_redo" group="edit_undo_merge" />
    <Action name="edit_cut" group="edit_paste_merge" />
    <Action name="edit_copy" group="edit_paste_merge" />
    <Action name="edit_paste" group="edit_paste_merge" />
    <Action name="edit_select_all" group="edit_select_merge" />
    <Action name="edit_find" group="edit_find_merge" />
    <Action name="edit_replace" group="edit_find_merge" />
    <Action name="go_goto_line" group="edit_find_merge"/>
  </Menu>

  <Menu name="view" noMerge="1"><text>&amp;View</text>
    <Action name="edit_expand_all_blocks" group="view_operations"/>
    <Action name="edit_collapse_all_blocks" group="view_operations" />  
  </Menu>

  <Menu name="tools" noMerge="1"><text>&amp;Tools</text>
<!--    <Action name="edit_indent" group="tools_operations"/>
    <Separator group="tools_operations"/> -->
    <Action name="tools_start_macro" group="tools_operations"/>
    <Action name="tools_stop_macro" group="tools_operations"/>
    <Action name="tools_execute_macro" group="tools_operations"/>
  </Menu>

  <Menu name="settings" noMerge="1"><text>&amp;Settings</text>
    <Action name="set_confdlg" group="configure_merge" />
  </Menu>
</MenuBar>

<ToolBar name="mainToolBar" noMerge="1"><text>Main Toolbar</text>
  <Action name="file_save" group="file_operations" />
  <Action name="file_save_as" group="file_operations" />
  <Action name="edit_undo" group="edit_operations" />
  <Action name="edit_redo" group="edit_operations" />
  <Action name="edit_cut" group="edit_operations" />
  <Action name="edit_copy" group="edit_operations" />
  <Action name="edit_paste" group="edit_operations" />
  <Action name="edit_find" group="find_operations" />
</ToolBar>

</kpartgui>
